<?php

namespace backend\modules\career\models;

use backend\models\user\UserSaveVacancy;
use backend\modules\district\models\District;
use backend\modules\doctor_questionnaire\models\DoctorQuestionnaire;
use backend\modules\dropdown_list\models\DropdownList;
use backend\modules\general_question\models\GeneralQuestion;
use backend\modules\subdistrict\models\Subdistrict;
use bedezign\yii2\audit\AuditTrailBehavior;
use common\models\User;
use yeesoft\behaviors\MultilingualBehavior;
use yeesoft\db\ActiveRecord;
use Yii;
use yii\behaviors\BlameableBehavior;
use yii\behaviors\SluggableBehavior;
use yii\behaviors\TimestampBehavior;

/**
 * This is the model class for table "vacancy".
 *
 * @property int $id
 * @property string $slug
 * @property int $status 0-pending,1-published
 * @property string $image_header
 * @property string $main_category
 * @property int $job_level_id
 * @property int $job_field_id
 * @property int $employee_status_id
 * @property int $location_id
 * @property int $schedule_id
 * @property int $published_at
 * @property int $expire_date
 * @property int $created_at
 * @property int $updated_at
 * @property int $created_by
 * @property int $updated_by
 * @property int $revision
 * @property string $view
 * @property string $layout
 *
 * @property District $location
 * @property DropdownList $jobLevel
 * @property DropdownList $jobField
 * @property DropdownList $schedule
 * @property DropdownList $employeeStatus
 * @property UserVacancy[] $userVacancies
 * @property User[] $users
 * @property VacancyLang[] $vacancyLangs
 */
class Vacancy extends ActiveRecord
{
    const STATUS_PUBLISHED = 1;

    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'vacancy';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['title', 'brief', 'content', 'job_field_id', 'job_level_id'], 'required'],
            [['status', 'job_level_id', 'job_field_id', 'employee_status_id', 'location_id', 'expire_date', 'created_at', 'updated_at', 'created_by', 'updated_by', 'revision'], 'integer'],
            [['slug', 'image_header', 'main_category', 'view', 'layout', 'brief'], 'string', 'max' => 255],
            ['published_at', 'date', 'timestampAttribute' => 'published_at', 'format' => 'yyyy-MM-dd'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            AuditTrailBehavior::className(),
            TimestampBehavior::className(),
            BlameableBehavior::className(),
            'sluggable' => [
                'class' => SluggableBehavior::className(),
                'ensureUnique' => true,
                'attribute' => 'title',
            ],
            'multilingual' => [
                'class' => MultilingualBehavior::className(),
                'langForeignKey' => 'parent_id',
                'tableName' => "{{%vacancy_lang}}",
                'attributes' => [
                    'title', 'content', 'brief'
                ]
            ],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('site', 'ID'),
            'slug' => Yii::t('site', 'Slug'),
            'status' => Yii::t('site', 'Status'),
            'image_header' => Yii::t('site', 'Image Header'),
            'main_category' => Yii::t('site', 'Main Category'),
            'job_level_id' => Yii::t('site', 'Level'),
            'job_field_id' => Yii::t('site', 'Section'),
            'employee_status_id' => Yii::t('site', 'Employee Status'),
            'location_id' => Yii::t('site', 'Location'),
            'schedule_id' => Yii::t('site', 'Schedule'),
            'published_at' => Yii::t('site', 'Published At'),
            'expire_date' => Yii::t('site', 'Expire Date'),
            'created_at' => Yii::t('site', 'Created At'),
            'updated_at' => Yii::t('site', 'Updated At'),
            'created_by' => Yii::t('site', 'Created By'),
            'updated_by' => Yii::t('site', 'Updated By'),
            'revision' => Yii::t('site', 'Revision'),
            'view' => Yii::t('site', 'View'),
            'layout' => Yii::t('site', 'Layout'),
        ];
    }


    public function getLocation()
    {
        return $this->hasOne(District::className(), ['id' => 'location_id']);
    }

    public function getEmployeeStatus()
    {
        return $this->hasOne(DropdownList::className(), ['id' => 'employee_status_id'])->andFilterWhere(['category' => DropdownList::EmployeeStatus]);
    }
    public function getJobField()
    {
        return $this->hasOne(DropdownList::className(), ['id' => 'job_field_id'])->andFilterWhere(['category' => DropdownList::JobTileField]);
    }
    public function getJobLevel()
    {
        return $this->hasOne(DropdownList::className(), ['id' => 'job_level_id'])->andFilterWhere(['category' => DropdownList::JobTileLevel]);
    }
    public function getSchedule()
    {
        return $this->hasOne(DropdownList::className(), ['id' => 'schedule_id'])->andFilterWhere(['category' => DropdownList::Schedule]);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUserVacancies()
    {
        return $this->hasMany(UserVacancy::className(), ['vacancy_id' => 'id'])->inverseOf('vacancy');
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUsers()
    {
        return $this->hasMany(User::className(), ['id' => 'user_id'])->viaTable('user_vacancy', ['vacancy_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getVacancyLangs()
    {
        return $this->hasMany(VacancyLang::className(), ['parent_id' => 'id'])->inverseOf('parent');
    }

    /**
     * @inheritdoc
     * @return VacancyQuery the active query used by this AR class.
     */
    public static function find()
    {
        return new VacancyQuery(get_called_class());
    }

    public function getAuthor()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    public function getPublishedDate()
    {
        return Yii::$app->formatter->asDate(($this->isNewRecord) ? time() : $this->published_at);
    }

    public function getCreatedDate()
    {
        return Yii::$app->formatter->asDate(($this->isNewRecord) ? time() : $this->created_at);
    }

    public function getUpdatedDate()
    {
        return Yii::$app->formatter->asDate(($this->isNewRecord) ? time() : $this->updated_at);
    }

    public function getPublishedTime()
    {
        return Yii::$app->formatter->asTime(($this->isNewRecord) ? time() : $this->published_at);
    }

    public function getCreatedTime()
    {
        return Yii::$app->formatter->asTime(($this->isNewRecord) ? time() : $this->created_at);
    }

    public function getUpdatedTime()
    {
        return Yii::$app->formatter->asTime(($this->isNewRecord) ? time() : $this->updated_at);
    }

    public function getPublishedDatetime()
    {
        return "{$this->publishedDate} {$this->publishedTime}";
    }

    public function getCreatedDatetime()
    {
        return "{$this->createdDate} {$this->createdTime}";
    }

    public function getUpdatedDatetime()
    {
        return "{$this->updatedDate} {$this->updatedTime}";
    }


    public function getRevision()
    {
        return ($this->isNewRecord) ? 1 : $this->revision;
    }

    public function updateRevision()
    {
        $this->updateCounters(['revision' => 1]);
    }

    public function hasGeneralQuestions()
    {
        //Check Main Category Only
        if (in_array($this->main_category, ['VACANCY_ADMIN', 'VACANCY_Allied_Health'])) {
            return GeneralQuestion::find()->where(['main_category' => $this->main_category, 'status' => 1])->count();
        }

        //Check Doctor Speciality
        if ($this->main_category == 'VACANCY_DOCTOR') {
            if (GeneralQuestion::find()->where([
                'main_category' => $this->main_category,
                'doctor_speciality_id' => $this->job_field_id,
                'status' => 1
            ])->count()){
                return 1;
            }
            else {
                //Check Doctor Category
                return GeneralQuestion::find()->where([
                    'main_category' => $this->main_category,
                    'doctor_speciality_id' => null,
                    'status' => 1
                ])->count();
            }
        }

    }

    public function getGeneralQuestions()
    {
        //Check Main Category Only
        if (in_array($this->main_category, ['VACANCY_ADMIN', 'VACANCY_Allied_Health'])) {
            return GeneralQuestion::find()->where(['main_category' => $this->main_category, 'status' => 1])->one();
        }

        //Check Doctor Speciality
        if ($this->main_category == 'VACANCY_DOCTOR') {
            if ($obj = GeneralQuestion::find()->where([
                'main_category' => $this->main_category,
                'doctor_speciality_id' => $this->job_field_id,
                'status' => 1
            ])->one()){
                return $obj;
            }
            else {
                //Check Doctor Category
                return GeneralQuestion::find()->where([
                    'main_category' => $this->main_category,
                    'doctor_speciality_id' => null,
                    'status' => 1
                ])->one();
            }
        }
    }




    public function hasDoctorQuestionnaire()
    {


        //Check Doctor Speciality
        if ($this->main_category == 'VACANCY_DOCTOR') {
            if (DoctorQuestionnaire::find()->where([
                'doctor_speciality_id' => $this->job_field_id,
                'status' => 1
            ])->count()){
                return 1;
            }
            else {
                //Check Doctor Category
                return DoctorQuestionnaire::find()->where([
                    'doctor_speciality_id' => null,
                    'status' => 1
                ])->count();
            }
        }

    }

    public function getDoctorQuestionnaire()
    {


        //Check Doctor Speciality
        if ($this->main_category == 'VACANCY_DOCTOR') {
            if ($obj = DoctorQuestionnaire::find()->where([
                'doctor_speciality_id' => $this->job_field_id,
                'status' => 1
            ])->one()){
                return $obj;
            }
            else {
                //Check Doctor Category
                return DoctorQuestionnaire::find()->where([
                    'doctor_speciality_id' => null,
                    'status' => 1
                ])->one();
            }
        }
    }

    public function save($runValidation = true, $attributeNames = null)
    {
        \Yii::$app->cache->flush();
        return parent::save($runValidation, $attributeNames);
    }

    public function isSaved()
    {
        return false;
//        return UserSaveVacancy::find()->where(['created_by' => Yii::$app->user->id, 'vacancy_id' => $this->id])->count();
    }
    public function isUserApplied()
    {
        return UserVacancy::find()->where(['user_id' => Yii::$app->user->id, 'vacancy_id' => $this->id])->count();
    }

    public function sendEmail($nationality)
    {
        if($nationality == 'Saudi, Saudi Arabian')
         Yii::$app->mailer->compose()
            ->setTo(Yii::$app->params['VacancyToEmailSaudi'])
            ->setFrom([Yii::$app->params['fromEmail'] => Yii::$app->params['fromName']])
            ->setSubject('Saudi')
            ->setTextBody('')
            ->send();
        else{
            Yii::$app->mailer->compose()
                ->setTo(Yii::$app->params['VacancyToEmailNonSaudi'])
                ->setFrom([Yii::$app->params['fromEmail'] => Yii::$app->params['fromName']])
                ->setSubject('Non Saudi')
                ->setTextBody('')
                ->send();
        }
        return true;
    }


}
